<?php 

namespace Quivers\Integration\Model;


use Magento\Payment\Model\CcGenericConfigProvider;
use Magento\Payment\Model\CcConfig;
use Stripe\Stripe;

/**
 * ConfigProvider Class
 */
class ConfigProvider extends CcGenericConfigProvider
{

	const CODE = 'quivers_integration';

    protected $scopeConfig;

	public function __construct(
	    CcConfig $ccConfig,
	    \Magento\Payment\Helper\Data $paymentHelper,
	    \Magento\Checkout\Model\Session $checkoutSession,
	    \Magento\Customer\Model\Session $customerSession,
	    \Magento\Payment\Model\Config $paymentConfig,
	    \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
	    \Magento\Framework\Message\ManagerInterface $messageManager,
	    array $methodCodes = []
	) {
	    // $this->paymentHelper    = $paymentHelper;
	    // $this->checkoutSession  = $checkoutSession;
	    $this->customerSession  = $customerSession;
	    // $this->paymentConfig    = $paymentConfig;
	    $this->scopeConfig = $scopeConfig;
	    $this->messageManager = $messageManager;
		$storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
	    parent::__construct($ccConfig, $paymentHelper, [static::CODE]);
	}

	/**
	 * Get checkout config.
	 *
	 * @return array
	 */
	public function getConfig()
	{
		$storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $publicKey = $this->scopeConfig->getValue('payment/quivers_integration/public_key', $storeScope);
        $title = $this->scopeConfig->getValue('payment/quivers_integration/title', $storeScope);
        $payPalTitle = $this->scopeConfig->getValue('payment/quivers_integration_paypal/title', $storeScope);
        $payPalClientId = $this->scopeConfig->getValue('payment/quivers_integration_paypal/client_id', $storeScope);
        $payPalSandbox = $this->scopeConfig->getValue('payment/quivers_integration_paypal/quivers_paypal_sanndbox', $storeScope);
        $active = $this->scopeConfig->getValue('quivers/parameters/active', $storeScope);
		$quiversSandbox = $this->scopeConfig->getValue('quivers/parameters/quivers_sanndbox', $storeScope);

		$customer = $this->customerSession->getCustomer();
		$customerId = $customer->getId();
		$stripeCustomerId = $customer->getQuiversStripeCustomerId();
		try{
		$customersList = \Stripe\Customer::all();
	
		foreach($customersList as $customerKey){
			$livemode = $customerKey->livemode;
			if($quiversSandbox == 1 && $livemode == true)
			{
				unset($customersList);
			}
		}
	
		if(empty($customersList)){
			$stripeCustomerId = '';
		}

		$stripeCards = [];
		$selectedCard = [];
		if($stripeCustomerId) {
			$stripeCustomer = \Stripe\Customer::retrieve($stripeCustomerId);
	        if($stripeCustomer) {
	        	$savedCardIds = array_map("trim", explode(',', $customer->getQuiversStripeCardIds()));
	        	if($stripeCustomer->id===$stripeCustomerId && $stripeCustomer->sources) {
	        		$cards = $stripeCustomer->sources->data;
	        		if(count($cards)>0) {
	        			foreach($cards as $card) {
	        				if(isset($card->card) && in_array($card->id, $savedCardIds)) {
	        					$details = [
		        					'paymentSourceId'=>$card->id,
		        					'last4' => $card->card->last4,
		        					'expMonth' => $card->card->exp_month,
		        					'expYear' => $card->card->exp_year,
		        					'country' => $card->card->country,
		        					'cardType' => $card->card->funding,
		        					'label' => "XXXX-XXXX-XXXX-".$card->card->last4.' '.$card->card->exp_month.'/'.$card->card->exp_year,
		        				];
		        				$stripeCards[] = $details;
		        				if($card->id===$stripeCustomer->default_source) {
		        					$selectedCard = $details;
		        				}
	        				}
	        			}
	        		}
	        	}
	        }
		}

		$config = parent::getConfig();
		$config = array_merge_recursive($config, [
			'validation' => $active ,
            'payment' => [
                static::CODE => [
					'publicKey' => $publicKey,
					'siteKey' =>isset($_SESSION['site_keys']['site_key'])?$_SESSION['site_keys']['site_key']:0,
                    'title'=> $title,
                    'payPalTitle' => $payPalTitle,
                    'payPalClientId' => $payPalClientId,
                    'payPalSandbox' => $payPalSandbox,
                    'stripeCards' => $stripeCards,
                    'selectedCard' => $selectedCard,
                    'customerId' => $customerId
                ],
            ],
        ]);

        return $config;
	}
	catch (\Exception $e) {
		$this->messageManager->addErrorMessage(__("Sorry! Looks like the Payment Method is not yet available to use because it has not yet been Fully Configured."));
        throw new \Magento\Framework\Validator\Exception(__('Stripe public key and secret keys are Invalid , Get these from your Stripe account API settings.'));}
}

}