<?php
namespace Quivers\Integration\Model;

use Stripe\Stripe;
use \Magento\Customer\Model\Session as customerSession;
use Quivers\Integration\LogTracking;

class QuiversStripe extends \Magento\Payment\Model\Method\Cc
{

    protected $_code = 'quivers_integration';
    protected $_canAuthorize = true;
    protected $_isGateway = true;
    protected $curlClient;
    protected $_stripeApi = false;
    protected $_scopeConfig;


    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Magento\Framework\Module\ModuleListInterface $moduleList,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        customerSession $customerSession
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $moduleList,
            $localeDate
        );
        $this->_scopeConfig = $scopeConfig;
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $this->curlClient = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\HTTP\Client\Curl::class);
        $this->_logger = $logger;
    
       
        // get the configured values in payment configuration form.
        $secret_key = $this->_scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);
        $this->_minAmount = $this->_scopeConfig->getValue("payment/quivers_integration/min_order_total", $storeScope);
        $this->customerSession = $customerSession;
        Stripe::setApiKey($secret_key);
    }

    public function assignData(\Magento\Framework\DataObject $data)
    {
        parent::assignData($data);
        $additionalData = $data->getAdditionalData();
        if (is_array($additionalData))
            $data->setData(array_merge($data->getData(), $additionalData));

        $info = $this->getInfoInstance();
    
        $token = $data['cc_stripejs_token'];
        $saveMyCard = $data['cc_save_card'];
        $captchatoken = $data['captcha_token'];
        // assigning data received from payment form so that it can be used
        // later in authorize method.
        $info->setAdditionalInformation('cc_stripejs_token', $token);
        $info->setAdditionalInformation('captcha_token', $captchatoken);
        $info->setAdditionalInformation('cc_save_card', $saveMyCard);
        return $this;
    }

   
    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        // get saved data
        $log = new LogTracking($this->_scopeConfig);
        $payment_token = $payment->getAdditionalInformation('cc_stripejs_token');
        $saveMyCard = $payment->getAdditionalInformation('cc_save_card');
        $token = $payment->getAdditionalInformation('captcha_token');
        $order = $payment->getOrder();
        $billing = $order->getBillingAddress();
        $customerEmail = $billing->getEmail();

        try {
            // check if user with same email already exists in Stripe account.
            $customer = false;
            $customersList = \Stripe\Customer::all(array(
                "limit" => 1,
                "email" => $customerEmail,
            ));

            // create a new user if it doesn't exists.
            if(count($customersList["data"])>0) {
                $customer = $customersList["data"][0];
            }

            if(!$customer){
                $customer = \Stripe\Customer::create(array(
                'source' => $payment_token,
                'email' => $customerEmail,
                ));
            }

            $customerId = $customer["id"];

            if ($saveMyCard)
            {  // checked
               try {
                   $customer->sources->create(array("source" => $payment_token));
                   }
               catch(\Exception $e){
                    }
            }
               $customerId = $customer["id"];

            $customer = $this->customerSession->getCustomer();
            $myCustomerId = $customer->getId();
            if(isset($customerId) && $myCustomerId) {
                $customer->setAttributeSetId(\Magento\Customer\Api\CustomerMetadataInterface::ATTRIBUTE_SET_ID_CUSTOMER);
                $customer->setQuiversStripeCustomerId($customerId);
                if ($saveMyCard) {  // checked
                    // add current card.
                    $savedCardIds = explode(',', $customer->getQuiversStripeCardIds());
                    $savedCardIds[] = $payment_token;
                    $customer->setQuiversStripeCardIds(implode(",", $savedCardIds));
                }
                $customer->save();
            }
        } catch (\Exception $e) {
            throw new \Magento\Framework\Validator\Exception(__($e->getMessage().' Please verify the details before trying again. If the issue persists please contact Quivers at 805-250-4555'));
        }
 
        $secret = isset($_SESSION['site_keys']['site_key_secret'])?$_SESSION['site_keys']['site_key_secret']:'NO-SECRET_KEY';

        if(isset($token) && $token!=null && isset($secret)):
            //get verify response data
            $verifyResponse = file_get_contents('https://www.google.com/recaptcha/api/siteverify?secret='.$secret.'&response='.$token.'&remoteip='.$_SERVER["REMOTE_ADDR"]);
            $responseData = json_decode($verifyResponse);
            $responseData = json_decode($verifyResponse);
            error_log('response='.json_encode($responseData));
            $log->captcha_data($token,$responseData->success);
                if($responseData->success):
                    $charge = $this->createCharge($customerId, $payment, $amount);
                else:
                    $responseData = json_decode(json_encode($responseData),true);
                    if(isset($responseData["error-codes"][0]) && $responseData["error-codes"][0]=='invalid-input-secret') :
                        $charge = $this->createCharge($customerId, $payment, $amount);
                    else:
                        $message = "Please reload your page  and try again. If the issue still persists,please contact 'enterprise@quivers.com' for further assistance";
                        throw new \Magento\Framework\Validator\Exception(__($message));
                    endif;
                endif;
        else:
            $charge = $this->createCharge($customerId, $payment, $amount);
        endif;

        $payment
            ->setTransactionId($charge->id)
            ->setIsTransactionClosed(0);
        
        $order_data = ["Customer Id"=>$customerId, "Email"=>$customerEmail];
        $response = ["Response"=>$charge, "Amount"=>$amount];
        $log->payment_request($order_data, "stripe", date('r', $charge["created"]), $response, $charge["status"]);
        return $this;
    }

    private function createCharge($customer, $payment, $amount) {
        try {
            $statement_descriptor_suffix = $payment->getAdditionalInformation('payment_descriptor');
            if(strlen($statement_descriptor_suffix) > 10){
                $statement_descriptor_suffix = substr($statement_descriptor_suffix, 0, 10);
            }
	        $order = $payment->getOrder();
            // charge amount should be grand total(converted price)
            // $amount = $order->getGrandTotal();
	        $amount = $amount;
            $currency = strtolower($order->getOrderCurrencyCode());
            $chargeAmount = $this->formatCurrencyAmount($amount, $currency);
            $request = array(
                'customer' => $customer,
                'source' => $payment->getAdditionalInformation('cc_stripejs_token'),
                'capture' => false,
                'amount' => $chargeAmount,
                'currency' => $currency,
                'description' => sprintf('#%s, %s', $order->getIncrementId(), $order->getCustomerEmail()),
                'statement_descriptor_suffix' => $statement_descriptor_suffix
            );
            if($payment->getAdditionalInformation('destination') === true){
                $merchantId = $payment->getAdditionalInformation('merchantId');
                if($merchantId != '0'){
                    $transferData = array(
                        "destination" => $merchantId,
                        "amount" => $chargeAmount
                    );
                    $onBehalfOf = $merchantId;
                    $request['transfer_data'] = $transferData;
                    $request['on_behalf_of'] = $onBehalfOf;
                }
            }
            $charge = \Stripe\Charge::create($request);
            return $charge;
        } catch (\Stripe\Error\ApiConnection $e) {
            // Network problem, perhaps try again.
            throw new \Magento\Framework\Validator\Exception(__('Unable to connect to payment gateway server. Please try again. If the issue persists please contact Quivers at 805-250-4555'));
        } catch (\Stripe\Error\InvalidRequest $e) {
            // You screwed up in your programming. Shouldn't happen!
            throw new \Magento\Framework\Validator\Exception(__('Invalid Request. Please try again. If the issue persists please contact Quivers at 805-250-4555'.$e->getMessage()));
        } catch (\Stripe\Error\Api $e) {
            // Stripe's servers are down!
            throw new \Magento\Framework\Validator\Exception(__('Unable to connect to payment gateway server. Please try again. If the issue persists please contact Quivers at 805-250-4555'));
        } catch (\Stripe\Error\Card $e) {
            // Card error.
            throw new \Magento\Framework\Validator\Exception(__($e->getMessage().' Please verify the details before trying again. If the issue persists please contact Quivers at 805-250-4555'));
        }

    }

    public function formatCurrencyAmount($amount, $currency)
    {

        // Stripe takes amounts as the smallest currency unit. ...

        if (empty($amount)) {
            return null;
        }
        $multiple = 100;
        if (in_array(
            $currency,
            array('BIF', 'DJF', 'JPY', 'KRW', 'PYG', 'VND', 'XAF', 'XPF', 'CLP', 'GNF', 'KMF', 'MGA', 'RWF', 'VUV', 'XOF'),
            true
        )) {
            $multiple = 1;
        }

        return sprintf('%01.0f', (float)$amount * $multiple);
    }



    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        if ($quote && $quote->getBaseGrandTotal() <= $this->_minAmount) {
            return false;
        }
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $secretKey = $this->_scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);
        $activekey = $this->_scopeConfig->getValue("payment/quivers_integration/active", $storeScope);
        $active = $this->_scopeConfig->getValue("quivers/parameters/active", $storeScope);
        if (!$secretKey  or $active==0) {
            return false;
        }

        return $activekey;
    }
    public function canUseForCurrency($currencyCode)
    {
        return true;
    }

    public function validate()
    {
        // validation is already handled by stripejs
        return true;
    }

}
